@@ -21,6 +21,12 @@ LaplaceApproximation <- function(Model=NULL, parm=NULL, Data=NULL,
           cat("Initial values were not supplied, and\n")
           cat("have been set to zero prior to LaplaceApproximation().\n")
           parm <- rep(0, length(Data$parm.names))}
+     for (i in 1:length(Data)) {
+          if(is.matrix(Data[[i]])) {
+               mat.rank <- qr(Data[[i]], tol=1e-10)$rank
+               if(mat.rank < ncol(Data[[i]])) {
+                    cat("WARNING: Matrix", names(Data)[[i]],
+                         "may be rank-deficient.\n")}}}
      if({Interval <= 0} | {Interval > 1}) Interval <- 1.0E-6
      Iterations <- round(Iterations)
      if(Iterations < 10) {Iterations <- 10}
