@@ -5,13 +5,13 @@
 ###########################################################################
 
 dbern <- function(x, prob, log=FALSE)
-     {dbinom(x, 1, prob, log)}
+     {return(dbinom(x, 1, prob, log))}
 pbern <- function(q, prob, lower.tail=TRUE, log.p=FALSE)
-     {pbinom(q, 1, prob, lower.tail, log.p)}
+     {return(pbinom(q, 1, prob, lower.tail, log.p))}
 qbern <- function(p, prob, lower.tail=TRUE, log.p=FALSE)
-     {qbinom(p, 1, prob, lower.tail, log.p)}
+     {return(qbinom(p, 1, prob, lower.tail, log.p))}
 rbern <- function(n, prob)
-     {rbinom(n, 1, prob)}
+     {return(rbinom(n, 1, prob))}
 
 ###########################################################################
 # Categorical Distribution                                                #
@@ -21,12 +21,12 @@ rbern <- function(n, prob)
 dcat <- function(x, p, log=FALSE)
      {
      if(!is.matrix(p)) p <- rbind(p)
-     if(is.vector(x) & (length(x) == 1)) {
+     if(is.vector(x) & {length(x) == 1}) {
           temp <- rep(0, ncol(p))
           temp[x] <- 1
           x <- t(temp)
           }
-     if(is.vector(x) & (length(x) > 1)) x <- indmat(x)
+     else if(is.vector(x) & (length(x) > 1)) {x <- indmat(x)}
      if(!identical(dim(x),dim(p))) stop("Dimensions of x and p differ in dcat().")
      dens <- x*p
      if(log == TRUE) dens <- x*log(p)
@@ -44,7 +44,7 @@ ddirichlet <- function(x, alpha, log=FALSE)
      {
      dirichlet1 <- function(x, alpha) {
           logD <- sum(lgamma(alpha)) - lgamma(sum(alpha))
-          s <- sum((alpha-1)*log(x))
+          s <- sum({alpha-1}*log(x))
           exp(sum(s)-logD)
           }
     if(!is.matrix(x)) x <- matrix(x)
@@ -80,23 +80,23 @@ rdirichlet <- function(n, alpha)
 dhalfcauchy <- function(x, scale=25, log=FALSE)
      {
      x <- as.vector(x)
-     if(scale <= 0) {stop("scale parameter negative in dhalfcauchy().\n")}
-     dens = 2*scale / (pi*(x^2 + scale^2))
+     if(scale <= 0) stop("scale parameter negative in dhalfcauchy().\n")
+     dens = 2*scale / (pi*{x*x + scale*scale})
      if(log == TRUE) dens <- log(dens)
      return(dens)
      }
 phalfcauchy <- function(q, scale=25)
      {
      q <- as.vector(q)
-     if(scale <= 0) {stop("scale parameter negative in phalfcauchy().\n")}
-     z <- (2/pi)*atan(q/scale)
+     if(scale <= 0) stop("scale parameter negative in phalfcauchy().\n")
+     z <- {2/pi}*atan(q/scale)
      return(z)
      }
 qhalfcauchy <- function(p, scale=25)
      {
      p <- as.vector(p)
-     if(scale <= 0) {stop("scale parameter negative in qhalfcauchy().\n")}
-     x <- scale*tan((pi*p)/2)
+     if(scale <= 0) stop("scale parameter negative in qhalfcauchy().\n")
+     x <- scale*tan({pi*p}/2)
      return(x)
      }
 rhalfcauchy <- function(n, scale=25)
@@ -104,7 +104,7 @@ rhalfcauchy <- function(n, scale=25)
      n <- as.vector(n)
      if(scale <= 0) {stop("scale parameter negative in rhalfcauchy().\n")}
      p <- runif(n, 0, 1)
-     x <- scale*tan((pi*p)/2)
+     x <- scale*tan({pi*p}/2)
      return(x)
      }
 
@@ -116,29 +116,29 @@ rhalfcauchy <- function(n, scale=25)
 dhalft <- function(x, scale=25, nu=1, log=FALSE)
      {
      x <- as.vector(x)
-     if(scale <= 0) {stop("scale parameter negative in dhalft().\n")}
-     dens = (1 + (1/nu)*(x/scale)^2)^(-(nu+1)/2)
+     if(scale <= 0) stop("scale parameter negative in dhalft().\n")
+     dens = (1 + {1/nu}*{x/scale}*{x/scale})^(-{nu+1}/2)
      if(log == TRUE) dens <- log(dens)
      return(dens)
      }
 phalft <- function(q, scale=25, nu=1)
      {
      q <- as.vector(q)
-     if(scale <= 0) {stop("scale parameter negative in phalft().\n")}
+     if(scale <= 0) stop("scale parameter negative in phalft().\n")
      z <- ptrunc(q, "st", a=0, b=Inf, mu=0, sigma=scale, nu=nu)
      return(z)
      }
 qhalft <- function(p, scale=25, nu=1)
      {
      p <- as.vector(p)
-     if(scale <= 0) {stop("scale parameter negative in qhalft().\n")}
+     if(scale <= 0) stop("scale parameter negative in qhalft().\n")
      x <- rtrunc(p, "st", a=0, b=Inf, mu=0, sigma=scale, nu=nu)
      return(x)
      }
 rhalft <- function(n, scale=25, nu=1)
      {
      n <- as.vector(n)
-     if(scale <= 0) {stop("scale parameter negative in rhalft().\n")}
+     if(scale <= 0) stop("scale parameter negative in rhalft().\n")
      x <- rtrunc(n, "st", a=0, b=Inf, mu=0, sigma=scale, nu=nu)
      return(x)
      }
@@ -152,14 +152,14 @@ rhalft <- function(n, scale=25, nu=1)
 dinvgamma <- function(x, shape, scale=1, log=FALSE)
      {
      # Initial Check
-     if(shape <= 0 | scale <=0) {
+     if({shape <= 0} | {scale <=0}) {
           stop("Shape or scale parameter negative in dinvgamma().\n")}
      alpha <- shape
      beta <- scale
      x <- as.vector(x)
      # done on log scale to allow for large alphas and betas
      log.density <- alpha * log(beta) - lgamma(alpha) -
-          (alpha + 1) * log(x) - (beta/x)
+          {alpha + 1} * log(x) - {beta/x}
      dens <- exp(log.density)
      if(log == TRUE) dens <- log.density
      return(dens)
@@ -177,22 +177,22 @@ rinvgamma <- function(n, shape, scale=1)
 dinvwishart <- function(Sigma, nu, R, log=FALSE)
      {
      if(!is.matrix(R)) R <- matrix(R)
-     if(nrow(R) != ncol(R)) {stop("Matrix R is not symmetric in dinvwishart().\n")}
+     if(nrow(R) != ncol(R)) stop("Matrix R is not symmetric in dinvwishart().\n")
      if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
-     if(nrow(Sigma) != ncol(Sigma)) {stop("Matrix Sigma is not symmetric in dinvwishart().\n")}
-     if(nrow(R) != ncol(Sigma)) {stop("Dimensions of Sigma and R differ in dinvwishart().\n")}
-     if(nu < nrow(R)) {stop("nu is less than the dimension of R in dinvwishart().\n")}
+     if(nrow(Sigma) != ncol(Sigma)) stop("Matrix Sigma is not symmetric in dinvwishart().\n")
+     if(nrow(R) != ncol(Sigma)) stop("Dimensions of Sigma and R differ in dinvwishart().\n")
+     if(nu < nrow(R)) stop("nu is less than the dimension of R in dinvwishart().\n")
      p <- nrow(R)
      # denominator
      gammapart <- 1
-     for (i in 1:p) {gammapart <- gammapart * gamma((nu + 1 - i)/2)}
-     denom <- gammapart *  2^(nu * p / 2) * pi^(p*(p-1)/4)
+     for (i in 1:p) {gammapart <- gammapart * gamma({nu + 1 - i} / 2)}
+     denom <- gammapart *  2^{nu * p / 2} * pi^{p*{p-1} / 4}
      # numerator
      detR <- det(R)
      detSigma <- det(Sigma)
-     hold <- R %*% solve(Sigma)
+     hold <- crossprod(R, solve(Sigma))
      tracehold <- sum(hold[row(hold) == col(hold)])
-     num <- detR^(nu/2) * detSigma^(-(nu + p + 1)/2) * exp(-1/2 * tracehold)
+     num <- detR^{nu/2} * detSigma^{-{nu + p + 1}/2} * exp(-1/2 * tracehold)
      dens <- num / denom
      if(log == TRUE) dens <- log(num / denom)
      return(dens)
@@ -209,46 +209,48 @@ rinvwishart <- function(nu, R) {return(solve(rwishart(nu,solve(R))))}
 dlaplace <- function(x, location=0, scale=1, log=FALSE)
      {
      x <- as.vector(x); location <- as.vector(location)
-     if(scale <= 0) {stop("scale parameter negative in dlaplace().\n")}
+     if(scale <= 0) stop("scale parameter negative in dlaplace().\n")
      logdens = (-abs(x - location) / scale) - log(2 * scale)
      if(log == FALSE) logdens <- exp(logdens)
+     return(logdens)
      }
 plaplace <- function(q, location=0, scale=1)
      {
      q <- as.vector(q); location <- as.vector(location)
-     if(scale <= 0) {stop("scale parameter negative in plaplace().\n")}
-     z = (q - location) / scale
+     if(scale <= 0) stop("scale parameter negative in plaplace().\n")
+     z = {q - location} / scale
      L = max(length(q), length(location), length(scale))
      q = rep(q, len=L)
      location = rep(location, len=L)
      scale = rep(scale, len=L)
-     ifelse(q < location, 0.5 * exp(z), 1 - 0.5 * exp(-z))
+     z <- ifelse(q < location, 0.5 * exp(z), 1 - 0.5 * exp(-z))
+     return(z)
      }
 qlaplace <- function(p, location=0, scale=1)
      {
      p <- as.vector(p); location <- as.vector(location)
-     if(scale <= 0) {stop("scale parameter negative in qlaplace().\n")}
+     if(scale <= 0) stop("scale parameter negative in qlaplace().\n")
      L = max(length(p), length(location), length(scale))
      p = rep(p, len=L)
      location = rep(location, len=L)
      scale = rep(scale, len=L)
-     location - sign(p - 0.5) * scale * log(2 * ifelse(p < 0.5,
-          p, 1 - p))
+     return(location - sign(p - 0.5) * scale * log(2 * ifelse(p < 0.5,
+          p, 1 - p)))
      }
 rlaplace <- function(n, location=0, scale=1)
      {
-     if(scale <= 0) {stop("scale parameter negative in rlaplace().\n")}
+     if(scale <= 0) stop("scale parameter negative in rlaplace().\n")
      location = rep(location, len=n)
      scale = rep(scale, len=n)
      r = runif(n)
-     location - sign(r - 0.5) * scale * log(2 * ifelse(r < 0.5,
-          r, 1 - r))
+     return(location - sign(r - 0.5) * scale * log(2 * ifelse(r < 0.5,
+          r, 1 - r)))
      }
 
 ###########################################################################
 # Multivariate Normal Distribution                                        #
 #                                                                         #
-# These functions are similar to those in the mnormt and mvtnorm          #
+# These functions are similar to those in the bayesm, mnormt, and mvtnorm #
 # packages.                                                               #
 ###########################################################################
 
@@ -258,24 +260,68 @@ dmvn <- function(x, mu, Sigma, log=FALSE)
      if(missing(Sigma)) Sigma <- diag(ncol(x))
      if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
      p <- nrow(Sigma)
-     ed <- eigen(Sigma, symmetric = TRUE)
-     ev <- ed$values
-     #if(!all(ev >= -1e-06 * abs(ev[1])))
-     #     stop("Sigma is not positive definite in dmvn().")
-     ss <- if(!is.matrix(mu)) {
-          x - rep(mu, each = nrow(x))
-          } else {
-               x - mu}
-     inv.Sigma <- ed$vectors %*% (t(ed$vectors)/ev)
-     quad <- 0.5 * rowSums((ss %*% inv.Sigma) * ss)
-     options(warn=-1)
-     if(is.nan(sum(log(ev)))) z <- log(1.0E-100) else z <- sum(log(ev))
-     options(warn=0)
-     fact <- -0.5 * (p * log(2 * pi) + z)
-     dens <- as.vector(fact - quad)
-     if(log == FALSE) dens <- exp(dens)
-     return(dens)
+     ev <- rep(-1,2) #for a check
+     posdeftest <- try(ed <- eigen(Sigma, symmetric = TRUE), silent=TRUE)
+     if(is.numeric(posdeftest[1])) ev <- ed$values
+     if(is.numeric(posdeftest[1]) & {all(ev >= -1e-06 * abs(ev[1]))}) {
+          #Sigma is positive definite
+          ss <- if(!is.matrix(mu)) {x - rep(mu, each = nrow(x))}
+               else {x - mu}
+          inv.Sigma <- crossprod(ed$vectors, t(ed$vectors)/ev)
+          quad <- 0.5 * rowSums({ss %*% inv.Sigma} * ss)
+          options(warn=-1)
+          if(is.nan(sum(log(ev)))) z <- log(1.0E-100) else z <- sum(log(ev))
+          options(warn=0)
+          fact <- -0.5 * {p * log(2 * pi) + z}
+          dens <- as.vector(fact - quad)
+          if(log == FALSE) dens <- exp(dens)
+          }
+     else if(is.character(posdeftest[1]) | {!all(ev >= -1e-06 * abs(ev[1]))}) {
+          #Sigma is not positive definite, so try...
+          if(is.vector(mu)) dens <- dmvn2(x, mu, Sigma, log=log)
+          if(is.matrix(mu)) {
+               dens <- rep(0,nrow(x))
+               rtest <- try(rooti <- backsolve(chol(Sigma), diag(ncol(x))),
+                    silent=TRUE)
+               if(is.numeric(rtest[1])) {for (i in 1:nrow(x)) {
+                    dens[i] <- dmvn3(x[i,], mu[i,], rooti, log=log)}}
+               else {for (i in 1:nrow(x)) {
+                    dens[i] <- dmvn2(x[i,], mu[i,], Sigma, log=log)}}
+               }
+          }
+     return(as.vector(dens))
      }
+#Won't accept mu as a matrix, so slowest row by row, but most reliable
+dmvn2 <- function(x, mu, Sigma, log=FALSE)
+     {
+     if(is.vector(x)) x <- matrix(x, ncol=length(x))
+     if(missing(Sigma)) Sigma <- diag(ncol(x))
+     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
+     k  <- if(is.matrix(Sigma)) ncol(Sigma) else 1
+     if(missing(mu)) mu <- rep(0,k)
+     if(!is.vector(mu)) stop("mu must be a vector in dmvn().")
+     mu <- matrix(mu, ncol=length(mu))
+     if(NCOL(mu) != NCOL(Sigma)) stop("Dimensions of mu and Sigma differ in dmvn().")
+     options(warn=-1)
+     distval <- mahalanobis(x, center=mu, cov=Sigma)
+     logdet <- sum(log(eigen(Sigma, symmetric=TRUE,
+          only.values=TRUE)$values))
+     logdens <- -{ncol(x)*log(2*pi) + logdet + distval} / 2
+     options(warn=0)
+     if(log == FALSE) logdens <- exp(logdens)
+     return(logdens)
+     }
+#Fastest way to process when both x and mu are matrices, but must be
+#done one by one.
+dmvn3 <- function(x, mu, rooti, log=FALSE)
+     {
+     z=as.vector(t(rooti)%*%(x-mu))
+     logdens <- -{length(x)/2} * log(2*pi) -.5*{z%*%z} +
+          sum(log(diag(rooti)))
+     if(log == FALSE) logdens <- exp(logdens)
+     return(logdens)
+     }
+#dmvn3(x=c(rep(0,2)),mu=c(rep(0,2)),rooti=backsolve(chol(Sigma),diag(2)))
 
 rmvn <- function(n=1, mu=rep(0,k), Sigma)
      {
@@ -297,21 +343,19 @@ dmvt <- function(x, mu, S, df=Inf, log=FALSE)
      p <- nrow(S)
      ed <- eigen(S, symmetric = TRUE)
      ev <- ed$values
-     #if (!all(ev >= -1e-06 * abs(ev[1])))
-     #    stop("S is not positive definite in dmvt().")
-     ss <- if(!is.matrix(mu)) {
-          x - rep(mu, each = nrow(x))
-          } else {
-               x - mu}
-     inv.S <- ed$vectors %*% (t(ed$vectors)/ev)
-     quad <- rowSums((ss %*% inv.S) * ss)/df
-     options(warn=-1)
+     if (!all(ev >= -1e-06 * abs(ev[1])))
+         stop("S is not positive definite in dmvt().")
+     ss <- if(!is.matrix(mu)) {x - rep(mu, each = nrow(x))}
+          else {x - mu}
+     inv.S <- crossprod(ed$vectors, t(ed$vectors)/ev)
+     quad <- rowSums({ss %*% inv.S} * ss) / df
+     #options(warn=-1)
      if(is.nan(sum(log(ev)))) z <- log(1.0E-100) else z <- sum(log(ev))
-     options(warn=0)
-     fact <- lgamma((df + p)/2) - lgamma(df/2) -
-          0.5 * (p * (log(pi) + log(df)) + z)
-     dens <- fact - 0.5 * (df + p) * log(1 + quad)
-     if(log == FALSE) dens <- exp(fact) * ((1 + quad)^(-(df + p)/2))
+     #options(warn=0)
+     fact <- lgamma({df + p}/2) - lgamma(df/2) -
+          0.5 * {p * {log(pi) + log(df)} + z}
+     dens <- fact - 0.5 * {df + p} * log(1 + quad)
+     if(log == FALSE) dens <- exp(fact) * {{1 + quad}^{-{df + p}/2}}
      return(dens)
      }
 
@@ -337,9 +381,9 @@ dst<-function(x, mu=0, sigma=1, nu=10, log=FALSE)
      if(any(nu <= 0))  stop("nu must be positive in dst().")
      if(length(nu) > 1) dens <- ifelse(nu > 1000000,
           dnorm(x, mu, sigma, log=FALSE),
-          (1/sigma) * dt((x-mu)/sigma, df=nu, log=FALSE))
-     else dens <- if(nu > 1000000) dnorm(x, mu, sigma, log=FALSE)
-     else (1/sigma) * dt((x-mu)/sigma, df=nu, log=FALSE)
+          {1/sigma} * dt({x-mu}/sigma, df=nu, log=FALSE))
+     else dens <- if(nu > 1000000) {dnorm(x, mu, sigma, log=FALSE)}
+          else {{1/sigma} * dt({x-mu}/sigma, df=nu, log=FALSE)}
      dens <- if(log == FALSE) dens else log(dens)
      return(dens)
      }
@@ -347,12 +391,13 @@ pst <- function(q, mu=0, sigma=1, nu=10, lower.tail=TRUE, log.p=FALSE)
      {
      if(any(sigma <= 0)) stop("sigma must be positive in pst().")
      if(any(nu <= 0)) stop("nu must be positive in pst().")
-     if(length(nu)>1) cdf <- ifelse(nu>1000000,
+     if(length(nu) > 1) cdf <- ifelse(nu > 1000000,
           pnorm(q, mu, sigma, lower.tail=lower.tail, log.p=log.p),
-          pt((q-mu)/sigma, df=nu,   lower.tail=lower.tail, log.p=log.p))
-     else cdf <- if(nu > 1000000) pnorm(q, mu, sigma,
-          lower.tail=lower.tail, log.p=log.p)
-     else pt((q-mu)/sigma, df=nu, lower.tail=lower.tail, log.p=log.p)
+          pt({q-mu}/sigma, df=nu, lower.tail=lower.tail, log.p=log.p))
+     else cdf <- if(nu > 1000000) {pnorm(q, mu, sigma,
+          lower.tail=lower.tail, log.p=log.p)}
+          else {pt({q-mu}/sigma, df=nu, lower.tail=lower.tail,
+               log.p=log.p)}
      return(cdf)
      }
 qst <- function(p, mu=0, sigma=1, nu=10, lower.tail=TRUE, log.p=FALSE)
@@ -360,12 +405,12 @@ qst <- function(p, mu=0, sigma=1, nu=10, lower.tail=TRUE, log.p=FALSE)
      if(any(sigma <= 0)) stop("sigma must be positive in qst().")
      if(any(nu <= 0)) stop("nu must be positive in qst().")
      if(any(p < 0)|any(p > 1)) stop("p must be between 0 and 1 in qst().")
-     if(length(nu)>1) q <- ifelse(nu > 1000000,
+     if(length(nu) > 1) q <- ifelse(nu > 1000000,
           qnorm(p, mu, sigma, lower.tail=lower.tail, log.p=log.p),
           mu + sigma * qt(p, df=nu, lower.tail=lower.tail))
-     else q <- if(nu > 1000000) qnorm(p, mu, sigma,
-          lower.tail=lower.tail, log.p=log.p)
-     else mu + sigma * qt(p, df=nu, lower.tail=lower.tail)
+     else q <- if(nu > 1000000) {qnorm(p, mu, sigma,
+          lower.tail=lower.tail, log.p=log.p)}
+          else {mu + sigma * qt(p, df=nu, lower.tail=lower.tail)}
      return(q)
      }
 rst <- function(n, mu=0, sigma=1, nu=10)
@@ -393,7 +438,7 @@ dtrunc <- function(x, spec, a=-Inf, b=Inf, ...)
      tt <- rep(0, length(x))
      g <- get(paste("d", spec, sep = ""), mode = "function")
      G <- get(paste("p", spec, sep = ""), mode = "function")
-     tt[x>=a & x<=b] <- g(x[x>=a&x<=b], ...) / (G(b, ...) - G(a, ...))
+     tt[x>=a & x<=b] <- g(x[x>=a & x<=b], ...) / (G(b, ...) - G(a, ...))
      return(tt)
      }
 
@@ -413,7 +458,7 @@ ptrunc <- function(x, spec, a=-Inf, b=Inf, ...)
      G <- get(paste("p", spec, sep = ""), mode = "function")
      tt <- G(apply(cbind(apply(cbind(x, bb), 1, min), aa), 1, max), ...)
      tt <- tt - G(aa, ...)
-     tt <- tt/(G(bb, ...) - G(aa, ...))
+     tt <- tt/{G(bb, ...) - G(aa, ...)}
      return(tt)
      }
 
@@ -423,7 +468,7 @@ qtrunc <- function(p, spec, a=-Inf, b=Inf, ...)
      tt <- p
      G <- get(paste("p", spec, sep = ""), mode = "function")
      Gin <- get(paste("q", spec, sep = ""), mode = "function")
-     tt <- Gin(G(a, ...) + p*(G(b, ...) - G(a, ...)), ...)
+     tt <- Gin(G(a, ...) + p*{G(b, ...) - G(a, ...)}, ...)
      return(tt)
      }
 
@@ -439,7 +484,7 @@ vartrunc <- function(spec, a=-Inf, b=Inf, ...)
      {
      if(a >= b) stop("Lower bound a is not less than upper bound b in vartrunc().")
      ex <- extrunc(spec, a = a, b = b, ...)
-     f <- function(x) (x - ex)^2 * dtrunc(x, spec, a = a, b = b, ...)
+     f <- function(x) {x - ex}^2 * dtrunc(x, spec, a = a, b = b, ...)
      tt <- integrate(f, lower = a, upper = b)$value
      return(tt)
      }
@@ -453,44 +498,42 @@ vartrunc <- function(spec, a=-Inf, b=Inf, ...)
 dwishart <- function(Omega, nu, S, log=FALSE)
      {
      if(!is.matrix(S)) S <- matrix(S)
-     if(nrow(S) != ncol(S)) {stop("Matrix Omega is not symmetric in dwishart()\n\n")}
+     if(nrow(S) != ncol(S)) stop("Matrix Omega is not symmetric in dwishart()\n\n")
      if(!is.matrix(Omega)) Omega <- matrix(Omega)
-     if(nrow(Omega) != ncol(Omega)) {stop("Matrix Omega is not symmetric in dwishart()\n\n")}
-     if(nrow(S) != ncol(Omega)) {stop("Dimensions of Omega and S differ in dwishart()\n\n")}
-     if(nu < nrow(S)) {stop("nu is less than the dimension of S in dwishart()\n\n")}
+     if(nrow(Omega) != ncol(Omega)) stop("Matrix Omega is not symmetric in dwishart()\n\n")
+     if(nrow(S) != ncol(Omega)) stop("Dimensions of Omega and S differ in dwishart()\n\n")
+     if(nu < nrow(S)) stop("nu is less than the dimension of S in dwishart()\n\n")
      p <- nrow(S)
      # denominator
      gammapart <- 1
-     for (i in 1:p) {gammapart <- gammapart * gamma((nu + 1 - i)/2)}
-     denom <- gammapart * 2^(nu * p / 2) * pi^(p*(p-1)/4)
+     for (i in 1:p) {gammapart <- gammapart * gamma({nu + 1 - i} / 2)}
+     denom <- gammapart * 2^{nu * p / 2} * pi^{p*{p-1}/4}
      # numerator
      detS <- det(S)
      detOmega <- det(Omega)
-     hold <- solve(S) %*% Omega
+     hold <- crossprod(solve(S), Omega)
      tracehold <- sum(hold[row(hold) == col(hold)])
-     num <- detS^(-nu/2) * detOmega^((nu - p - 1)/2) * exp(-1/2 * tracehold)
+     num <- detS^(-nu/2) * detOmega^{{nu - p - 1}/2} * exp(-1/2 * tracehold)
      dens <- num / denom
-     if (log == TRUE) dens <- log(num / denom)
+     if(log == TRUE) dens <- log(num / denom)
      return(dens)
      }
 
 rwishart <- function(nu, S)
      {
      if(!is.matrix(S)) S <- matrix(S)
-     if(nrow(S) != ncol(S)) {stop("Matrix S is not symmetric in rwishart().\n")}
+     if(nrow(S) != ncol(S)) stop("Matrix S is not symmetric in rwishart().\n")
      if(nu < nrow(S)) {
           stop("nu is less than the dimension of S in rwishart().\n")}
      p <- nrow(S)
-     CC <- chol(S)
      Z <- matrix(0, p, p)
-     diag(Z) <- sqrt(rchisq(p, nu:(nu-p+1)))
-     if(p > 1)
-          {
+     diag(Z) <- sqrt(rchisq(p, nu:{nu-p+1}))
+     if(p > 1) {
           pseq <- 1:(p-1)
           Z[rep(p*pseq, pseq) +
-               unlist(lapply(pseq, seq))] <- rnorm(p*(p-1)/2)
+               unlist(lapply(pseq, seq))] <- rnorm(p*{p-1}/2)
           }
-     return(crossprod(Z %*% CC))
+     return(crossprod(Z %*% chol(S)))
      }
 
 #End
